﻿#--------------------------------------------
# Declare Global Variables and Functions here
#--------------------------------------------

#region Functions

function Change-RegValue
{
	param
	(
		[Parameter(Mandatory = $true,
					  Position = 0)]
		[string]$RegPath,
		[Parameter(Mandatory = $true,
					  Position = 1)]
		[string]$RegKey,
		[Parameter(Mandatory = $true,
					  Position = 2)]
		[string]$Value
	)
	
	Set-ItemProperty -Path $RegPath -Name $RegKey -Value $Value
}


function Format-Message($errData)
{
	$errObj = [PSCustomObject] @{
		"Text"	      = ""
		"Program"	  = ""
		"Line"	      = ""
		"Full"	      = ""
	}
	
	$errObj.Text = $errData.Exception.Message
	$scriptName = $errData.InvocationInfo.ScriptName
	$errObj.Program = $scriptName.Substring(($scriptName.LastIndexOf('\')) + 1)
	$errObj.Line = $errData.InvocationInfo.ScriptLineNumber
	$errObj.Full = "$($errObj.Text)`rProgram:$($errObj.Program)`rLine: $($errObj.Line)`r"
	Write $errObj
}

function Get-AppCfgValue($key, $websitePath)
{
	[xml]$xml = Get-Content "$websitePath\appSettings.config"
	foreach ($_ in $xml.appsettings.add) #Search through the appsettings.add nodes
	{
		if ($_.key -eq $key)
		{ return $_.value }
	}
	return $false
}

function Get-FiconfigInfo($Path)
{
	$xml = New-Object -TypeName XML
	$xml.Load("$Path\connectionstrings.config")
	$node = $xml.SelectSingleNode("//connectionStrings/add")
	$Cnnstring = $node.connectionString
	$split = $Cnnstring.split("=")
	$returnObj = New-Object psobject -Property @{
		SqlInstance	    = ($split[1].Split(";")[0])
		FiConfigDb	    = ($split[2].Split(";")[0])
		UserName	    = ($split[3].Split(";")[0])
		Password	    = ($split[4].Split(";")[0])
		DbName		    = ''
		Institution	    = ''
	}
	$query = "SELECT [BankClassicConnectionString], [Name]  FROM [$($returnobj.FiConfigDb)].[dbo].[Bank]"
	$dataRow = (Invoke-Query -DbName $returnObj.FiConfigDb -Qry $query)
	$returnObj.DbName = ($dataRow.BankClassicConnectionString.Split(';'))[1].split('=')[1]
	$returnObj.Institution = $dataRow.Name
	return $returnObj
}

function Get-Websites
{
<#	Try
	{
#>	$siteObjects = @()
	$allsites = @()
	$allSites = gci iis:\sites | Where-Object -FilterScript { $_.name -notlike "Default*" }
	$allSites | ForEach-Object {
		$website = $_
		$websitePath = $_.physicalPath
		if (!(Test-Path ($_.physicalPath + '\connectionstrings.config')))
		{
			Throw "ConnectionsString.config file not found in $($_.physicalPath)"
		}
		$siteObj = New-Object PSCustomObject -Property ([Ordered] @{
				CurrentVersion	= ''
				CsiServer		= ''
				DbName			= ''
				Institution		= ''
				SSL			   = ''
				SqlInstance		= ''
				WebSite			= $website.name
				WebSitePath		= $websitePath
			})
		$siteObj.CurrentVersion = (gci "$websitePath\bin\OnBoardSharedServices.dll").VersionInfo.fileversion
		$CsiServer = Get-AppCfgValue -key CSiJobManagerUri -webSitePath $websitePath
		$siteObj.CsiServer = ($CsiServer.split(':'))[1] -replace '//', ''
		$dbInfo = Get-FiconfigInfo -Path $websitePath
		$siteObj.DbName = $dbInfo.DbName
		$siteObj.Institution = $dbInfo.Institution
		$siteObj.SqlInstance = $dbInfo.SqlInstance
		if ($website.bindings.collection.protocol -eq 'https') { $siteObj.ssl = 'Y' }
		elseif ($website.bindings.collection.protocol -eq 'http') { $siteObj.ssl = 'N' }
		
		$siteObjects += $siteObj
	}
	$siteObjects | ForEach-Object { Populate-UpdSiteInfo $siteObjects }
	
	
<#	}
	catch { Show-Msg -msgName Error -msgText (Format-Message $_).full }
	#>
}

function Get-OnboardRegInfo
{
	$regInfo = Get-Item -Path HKLM:\software\Wow6432Node\OnboardDeposits
	$regInfoObj = [PSCustomObject] @{
		DbName	      = $regInfo.GetValue('DbName');
		DbUser	      = $regInfo.GetValue('DbUser');
		DbPassword    = $regInfo.GetValue('DbPassword');
		MultiBank	  = $regInfo.GetValue('MultiBank');
		SqlInstance   = $regInfo.GetValue('SqlInstance');
		UpdateFolder  = $regInfo.GetValue('UpdateFolder')
	}

	return $regInfoObj
}

function Get-ScriptDirectory
{
<#
	.SYNOPSIS
		Get-ScriptDirectory returns the proper location of the script.

	.OUTPUTS
		System.String
	
	.NOTES
		Returns the correct path within a packaged executable.
#>
	[OutputType([string])]
	param ()
	if ($null -ne $hostinvocation)
	{
		Split-Path $hostinvocation.MyCommand.path
	}
	else
	{
		Split-Path $script:MyInvocation.MyCommand.Path
	}
}

function Invoke-Query
{
	param
	(
		[Parameter(Mandatory = $true)]
		[string]$Qry,
		[string]$DbName = $regInfoObj.DbName
	)
	
	Invoke-Sqlcmd -Database $DbName -ServerInstance $regInfoObj.SqlInstance -Query $qry -ConnectionTimeout 5 -OutputSqlErrors $true
}

function Populate-UpdSiteInfo($siteObjects)
{
	$query = "Truncate Table dbo.UpdateInfo"
	Invoke-Query -Qry $query
	
	foreach ($obj in $siteObjects)
	{
		$query = @"
						USE DepositsMultibank
						GO
						Insert Into dbo.UpdateInfo
						(
							WebSite,
							Institution,
							CsiServer,
							CurrentVersion,
							DbName,
							SSL,
							WebSitePath,
							SqlInstance
						)
						VALUES
						(
							'$($obj.WebSite)',
							'$($obj.Institution)',
							'$($obj.CsiServer)',
							'$($obj.CurrentVersion)',
							'$($obj.DbName)',
							'$($obj.SSL)',
							'$($obj.WebSitePath)',
							'$($obj.SqlInstance)'
						)
"@
		
		Invoke-Query -DbName Master  -Qry $query
	}
}

function Recycle-Item
{
	param ([string]$Path)
	
	$shell = New-Object -ComObject 'Shell.Application'
	
	$shell.NameSpace(0).
	ParseName($Path).
	InvokeVerb('Delete')
}

function Show-Msg($msgName, $msgText, $msgType)
{
	Add-Type -AssemblyName PresentationCore, PresentationFramework
	If ($msgType -eq 'YesNo') { [System.Windows.MessageBox]::Show($msgText, $msgName, 'YesNo', 'Question') }
	Else { [System.Windows.MessageBox]::Show($msgText, $msgName) }
}

function Update-DB
{
	
}

function Write-Log ($updateObj)
{
	add-content (Get-Date -Format T).toString() -Path $updateObj.LogFilePath
	add-content "$($updateObj.LogData) `n--------------------------------------------------------------------------------------------------------------------------------" -Path $updateObj.LogFilePath
	write $updateObj
}
#endregion Functions

#region GlobalVariables
$ErrorActionPreference = 'Stop'
$script:scriptDirectory = Get-ScriptDirectory
$script:mgrVersion = (GCI "C:\Program Files\Jack Henry Onboard\Deposits Multi-Bank Update Manager\Deposits Multi-Bank Update Manager.exe").VersionInfo.FileVersion
$script:regInfoObj = Get-OnboardRegInfo
#endregion GlobalVariables



